
import sqlite3 as db
import time
import es

conn = None
cur = None
cache = None

info = es.AddonInfo()
info.name = 'Preferences'
info.version = '1.0'
info.author = 'AdminC'


    
class PrefException(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return self.value

class PlayerPref(dict):
    
    def __init__(self, plr_id):
        self.plr_id = plr_id
        
    def __getitem__(self, key):
        data = getPref(self.plr_id, key)
        return data['value']
        
    def __setitem__(self, key, value):
        setPref(self.plr_id, key, value)
        
    def __contains__(self, key):
        return (0 != self.timeof(key) )

    def timeof(self, key):
        return getPref(self.plr_id, key)['time']
    

        

class Cache(object):
    def __init__(self):
        self.plrs  = {}
        self.types = {}
        
        
    def getType(self, key):
        if key in self.types:
            return self.types[key]
        
        cur.execute("SELECT * FROM pref_types WHERE pref_key = ?;", (key,) )
        
        data = cur.fetchone()
        pref_type = {}
        if data != None:
            for i in range(len(cur.description ) ):
                pref_type [ cur.description[i][0] ] = data[i]
            
            self.types[ pref_type['pref_key'] ] = pref_type
            
            return pref_type
            
        else:
            raise PrefException("No such pref_type '%s'!" % key)
    

    def setType(self, key, cast = str, perm = 3, default = None):
        if key in self.types:
            cur.execute("UPDATE pref_types SET pref_perm = ? WHERE pref_key = ?;", (perm, key) )
            conn.commit()
            self.types[key]['pref_perm'] = perm
            return
        print key, perm
        cur.execute("INSERT INTO pref_types (pref_key, pref_perm) VALUES (?, ?);", (key, perm) )
        conn.commit()
        
        self.getType(key)
        self.types[key]['pref_cast'] = cast
        self.types[key]['pref_default'] = default

    def registerType(self, key, cast = str, perm = 3, default = None):
        self.setType(key, cast, perm, default)
        
    def getPlayer(self, key):
        if key in self.plrs:
            return self.plrs[key]
            
        cur.execute("SELECT * FROM pref_plrs WHERE  plr_key = ?;", (key,) )
        
        data = cur.fetchone()
        
        plr = {}
        
        if data != None:
            for i in range(len(cur.description ) ):
                plr[ cur.description[i][0] ] = data[i]
            
            self.plrs[ plr['plr_key'] ] = plr
            
            return plr
            
        else:
            cur.execute("INSERT INTO pref_plrs (plr_key) VALUES (?);", (key, ) )
            conn.commit()
            
            return self.getPlayer(key)
        


CREATE_PLAYER_TABLE = "CREATE TABLE IF NOT EXISTS pref_plrs (plr_id INTEGER PRIMARY KEY AUTOINCREMENT, plr_key VARCHAR(24) NOT NULL );"
CREATE_TYPE_TABLE   = "CREATE TABLE IF NOT EXISTS pref_types (pref_id INTEGER PRIMARY KEY AUTOINCREMENT, pref_key VARCHAR(32) NOT NULL, pref_perm INTEGER NOT NULL);"
CREATE_PREF_TABLE   = "CREATE TABLE IF NOT EXISTS pref_table (plr_id INTEGER NOT NULL, pref_id INTEGER NOT NULL, pref_value VARCHAR(255) NOT NULL, pref_time INTEGER NOT NULL, PRIMARY KEY (plr_id, pref_id) );"





def load():
    global conn, cur, cache
    conn = db.connect(es.getAddonPath("prefs") + "/prefs.db")
    cur = conn.cursor()
    
    initTables()
    
    cache = Cache()

def unload():
    conn.close()
    
    
    
def initTables():
    cur.execute(CREATE_PLAYER_TABLE )
    cur.execute(CREATE_TYPE_TABLE )
    cur.execute(CREATE_PREF_TABLE )
    conn.commit()

def getPlayer(userid):
    """
    This is a convenient wrapper class which allows you to more extensively deal with prefs.  
    This function returns a PrefPlayer Object, which basically acts like a dictionary.  You
    can access the prefs like so:
        player = prefs.getPlayer(userid)
        player["mypref"] = 9
        player["mypref"] += 28
        player["mypref"] = player["mypref"] - 37
        
        if "mypref" in player:
            print "You have this preference already set."
            "lolcount" in
    @param userid The userid of this wrapper
    @return PlayerPref
    
    Design Note:  If you try to do things like 'for each in player:' nothing will happen.  
    Im not sure what immeadiate use this has, but this plugin should be easy enough to add
    this kind of functionality.            
       
    """
    
    return PlayerPref(userid)

def register(key, cast = str, perm = 3, default = None):
    """
    Registers a preference type.  Preference types must be registered or they cannot be accessed.
    You only have to register a pref type once, but doing so more than once will do no harm.
    
    @param key This is the name of the preference you are registering.
    @param perm This is the permission for this type. 
    @param cast This is the datatype of what is being stored.
    @param default The default value to return, incase the player does not have the pref.
    @return None
    
    Perm types:
        0 - No read access, no write access
        1 - Read access, no write access
        2 - No read access, write access
        3 - Read access, write access. (default)
        
    Cast:
        This is a function that all pref values are filtered through.
        For example, if your pref is a floating point value, pass it the 'float' function.
        The default is to use the 'str' function.
        
    Design note: I considered making the pref type being created automatically if it didn't exist, 
    (IE, the PHP way) but I think this would cause more confusion if a typo were made in the pref
    name. (the bug vs. feature argument).
    
    Design note: Perm types are only guidelines and are not actually enforced.  This is up to the 
    plugin writer.  The access perms are for the players themselves, not the plugins.
    
    Design note: The default values and cast functions are not stored in the database, and can be 
    changed on the fly, even if the pref is already registered.
    """
    
    cache.registerType(key, cast, perm, default)


def getPref(userid, key):
    """
    Grabs the preference of the player by his userid.  If the player has no preference set, None is
    returned.  The preference must be registered before it can be accessed at all.  
    
    @param userid The userid of the player
    @param key The name of the preference to get
    @return dict
    
    Returns a dictionary with the following keys:
        value (str) - The value of the preference
        time  (int) - The timestamp the preference was last modified
    """
    plr_key = es.createplayerlist(userid)[int(userid)]["steamid"]
    plr = cache.getPlayer(plr_key)
    
    pref_type = cache.getType(key)
    
    cur.execute("SELECT * FROM pref_table WHERE plr_id = ? AND pref_id = ?;" , (plr['plr_id'], pref_type['pref_id']) )
    
    data = cur.fetchone()
    
    if data != None:
        value = pref_type['pref_cast'](data[2])
        thetime = int(data[3])
    else:
        value = pref_type['pref_default']
        thetime = 0

    return {'value': value, 'time':thetime}

def setPref(userid, key, value):
    """
    Sets the preference for the player.  The preference must be registered before it can be accessed 
    at all.  
    
    @param userid The userid of the player
    @param key The name of the preference to set
    @param value The value of the preference to set
    @return None
    
    """
    
    plr_key = es.createplayerlist(userid)[int(userid)]["steamid"]
    plr = cache.getPlayer(plr_key)
    
    pref_type = cache.getType(key)
    
    cur.execute("REPLACE INTO pref_table (plr_id, pref_id, pref_value, pref_time) VALUES (?, ?, ?, ?);",
                (plr['plr_id'], pref_type['pref_id'], pref_type['pref_cast'](value), int(time.time() ) )
               )
               
          
    conn.commit()



